      program pftsearch

C     *************************************************************************
C     *                                                                       *
C     *                     *****     pftsearch.f     *****                   *
C     *                                                                       *
C     * MODEL-BASED PROCEDURE TO DETERMINE ORIGINS, ORIENTATIONS, AND         *
C     *  MAG SCALE FACTORS OF IMAGED PARTICLES.  IT CAN BE USED TO FIND OR    *
C     *  REFINE THESE PARAMETERS.                                             *
C     *                                                                       *
C     * PROJECT CUBIC (NCOL=NROW=NSEC) 3D MAP (DEFAULT ASSUME 532 SYMMETRY    *
C     *  IN REGULARLY SPACED THETA,PHI ORIENTATIONS FOR A USER-DEFINED        *
C     *  REGION OF 3D SPACE (DEFAULT IS ONE-HALF OF THE ASYMMETRIC UNIT).     *
C     *  THESE CARTESIAN PROJECTIONS ARE THEN CONVERTED TO POLAR MAPS AND     *
C     *  POLAR FFTS.                                                          *
C     *                                                                       *
C     * LOGICAL UNITS:                                                        *
C     *           1 = REFERENCE 3D MAP (READ IF MODE.GT.0)                    *
C     *           2 = INPUT PARAMETERS DATA FILE(s)                           *
C     *           3 = PRJ DATABASE FILE (DEFAULT = PFT.PRJS) (IF MODE.LT.0)   *
C     *           4 = PFT DATABASE FILE (DEFAULT = PFT.PFTS) (IF MODE.LT.0)   *
C     *           7 = INPUT RAW IMAGE DATA                                    *
C     *           8 = STATS OUTPUT FILE (EMPFT.RES1)                          *
C     *           9 = STATS OUTPUT FILE (EMPFT.RADS)                          *
C     *          10 = STATS OUTPUT FILE (EMPFT.RES2)                          *
C     *          11 = PARAMETER OUTPUT DATA FILES (EMPFT.NEW_DAT)             *
C     *                                                                       *
C     * May 2003 - program inception. Program modified from old pft.f         *
C     *                                                                       *
C     *************************************************************************
C     
      IMPLICIT NONE

      include 'mpif.h'
      include '../include/allprogs.inc'

      COMMON/PFT_COM/ MODE, BIN, VERBOSE, MAG_CEN, MAG_STEP, 
     $     MAG_NUM, MAG_NORM, XCEN_MODEL, YCEN_MODEL, SYM, DTHE, 
     $     DPHI_90, PFTRAD_LO, PFTRAD_HI, JCUT, PIXRES_MIN, PIXRES_MAX, 
     $     ANN_LO, ANN_HI, VAROPT, SIGCUT, RES_MIN, RES_MAX, IDIM1, 
     $     IDIM2, RADIUS, DEL_ANG, NVIEW_MOD, NTHE, NPHI, 
     $     PARAM_FNAMES, IMG_FNAMES

c      COMMON/CTF_COM/PIXSIZ, UNITS, VOLTS, AMP_FAC, FOCUS_MAJ, 
c     1 FOCUS_MIN, ANG_MAJ, CS_COEFF

c      real*4    PIXSIZ(999), VOLTS(999), AMP_FAC(999)
c      real*4    FOCUS_MAJ(999), FOCUS_MIN(999), ANG_MAJ(999)
c      real*4    CS_COEFF(999), PIXSIZE
c      integer*4 UNITS(999)

      integer*4 BIN, VERBOSE, MODE, NFILES, COMPARE_MODE
      integer*4 SYM, NCOL, NROW, NSEC, NROT, NANNULI, IDIM1, IDIM2
      integer*4 NTHE, NPHI, NVIEW_MOD, NPIX, JCUT, ANN_LO, ANN_HI
      integer*4 NIDS_TOT, NIDS(999), NCOLBIN, NROWBIN, NSECBIN
      integer*4 MAG_NUM, MAG_NORM, alloc_stat, mcw
      real*4    DTHE, DPHI_90, DEL_ANG, MAG_CEN, MAG_STEP
      real*4    XCEN_MODEL, YCEN_MODEL, RES_MIN, RES_MAX, FILT_FAC(10)
      real*4    PFTRAD_LO, PFTRAD_HI, PIXRES_MIN, PIXRES_MAX
      real*4    RADIUS, SIGCUT, PI
      CHARACTER PARAM_FNAMES(999)*256, IMG_FNAMES(999)*256
      LOGICAL   VAROPT
      
      integer, allocatable :: FIDS(:), IDS(:)
      REAL, allocatable  :: TS(:), PS(:), OS(:), XS(:), YS(:), MAGS(:)
      REAL, allocatable  :: PFT_CCS(:), PRJ_CCS(:), CMP_CCS(:)
C      -------------------------------------------------------------

      call mpi_init(mpi_err)
      if(mpi_err .ne. 0) then
         call error_stop('pftsearch', 'main', 'Err in mpi_init')
      endif
      call mpi_comm_rank(mpi_comm_world, iam, mpi_err)
      if(mpi_err .ne. 0) then
         call error_stop('pftsearch', 'main', 'Err in mpi_comm_rank')
      endif
      call mpi_comm_size(mpi_comm_world, num_nodes, mpi_err)
      if(mpi_err .ne. 0) then
         call error_stop('pftsearch', 'main', 'Err in mpi_comm_size')
      endif

      pi = acos(-1.0)

c     ------------------------ MASTER -------------------------------
      if(iam .eq. 0) then

         PRINT *, 'PFTsearch: Model-based PFT analysis'
         PRINT *, 'Parallel version run on ', num_nodes , ' CPUs'
         PRINT *, 'For documentation see: http://bilbo.bio.purdue.edu'
         PRINT *, '/~baker/programs/doc/current/empft.pdf'
         PRINT *
         
C     Get user input
         call key_info (ncol, nrow, nsec, ncolbin, nrowbin, nsecbin,
     $        nrot,nannuli, npix, nfiles, nids, nids_tot,
     $        compare_mode ,filt_fac)
                  
      endif

c     -------------------- EVERYBODY -------------------------------
C     Distribute data to all processes
      mcw = mpi_comm_world ! Abbreviation

      call mpi_bcast(ncol,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nrow,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nsec,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(ncolbin,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nrowbin,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nsecbin,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nrot,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nannuli,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(npix,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nfiles,         1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nids,         999, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nids_tot,       1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(compare_mode,   1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(filt_fac,      10, mpi_real,    0, mcw, mpi_err)

      allocate(FIDS(NIDS_TOT), IDS(NIDS_TOT), TS(NIDS_TOT),
     $     PS(NIDS_TOT), OS(NIDS_TOT), XS(NIDS_TOT), YS(NIDS_TOT),
     $     MAGS(NIDS_TOT), PFT_CCS(NIDS_TOT), PRJ_CCS(NIDS_TOT),
     $     CMP_CCS(NIDS_TOT), stat = alloc_stat)
      if(alloc_stat .gt. 0) then
         write(*,*) "Memory allocation failed in PFTSEARCH"
         stop
      endif

      call mpi_bcast(mode,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(bin,            1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(verbose,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(mag_cen,        1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(mag_step,       1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(mag_num,        1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(mag_norm,       1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(xcen_model,     1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(ycen_model,     1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(sym,            1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(dthe,           1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(dphi_90,        1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(pftrad_lo,      1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(pftrad_hi,      1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(jcut,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(pixres_min,     1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(pixres_max,     1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(ann_lo,         1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(ann_hi,         1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(varopt,         1, mpi_logical, 0, mcw, mpi_err)
      call mpi_bcast(sigcut,         1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(res_min,        1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(res_max,        1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(idim1,          1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(idim2,          1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(radius,         1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(del_ang,        1, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(nview_mod,      1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nthe,           1, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(nphi,           1, mpi_integer, 0, mcw, mpi_err)

      if (iam .eq. 0) then
C     Fill particle parameter arrays
         CALL FILL_PARAMS (MODE, NFILES, NIDS_TOT, NIDS, FIDS, IDS, TS,
     $        PS, OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS,
     $        PARAM_FNAMES)
      endif

      call mpi_bcast(fids,    nids_tot, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(ids,     nids_tot, mpi_integer, 0, mcw, mpi_err)
      call mpi_bcast(ts,      nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(ps,      nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(os,      nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(xs,      nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(ys,      nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(mags,    nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(pft_ccs, nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(prj_ccs, nids_tot, mpi_real,    0, mcw, mpi_err)
      call mpi_bcast(cmp_ccs, nids_tot, mpi_real,    0, mcw, mpi_err)

C     GLOBAL MODE only 
      IF(MODE.NE.3) THEN
         CALL GLOBAL (NCOLBIN, NROWBIN, NSECBIN, 
     $        NROT, NANNULI, NIDS_TOT, FIDS, IDS, TS, 
     $        PS, OS, XS, YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, 
     $        COMPARE_MODE, FILT_FAC)
      ELSE
         print *, '****************************************************'
         print *, '*  Refine mode not supported by pftsearch program. *'
         print *, '****************************************************'
         call tsbend
      ENDIF

c     ------------------------ MASTER -------------------------------
      if(iam .eq. 0) then
         CALL WRITE_PARAMS (NIDS_TOT, FIDS, IDS, TS, PS, OS, XS, 
     $        YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS, PARAM_FNAMES, 
     $        IMG_FNAMES, MAG_NORM)

         CLOSE (UNIT=2)		! LAST INPUT PARAMETER FILE
         CLOSE (UNIT=11)        ! LAST OUTPUT PARAMETER FILE
         CALL PIF_CLOSE (7)     ! LAST INPUT IMAGE FILE
         IF(MODE.EQ.-1 .OR. MODE.EQ.-2) THEN
            CLOSE (UNIT=3)
            CLOSE (UNIT=4)
         ENDIF
         
C     Calculate average CCs and MAG factor if NIDS_TOT > 0
         CALL CALC_AVGS (NIDS_TOT, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS)
         IF(VERBOSE.GE.1) THEN
            PRINT *,'=========================================='
            PRINT *,' Files EMPFT.RADS, EMPFT.RES1, EMPFT.RES2 '
            PRINT *,'  written to default directory.'
            PRINT *,'=========================================='
            CLOSE (UNIT= 8)
            CLOSE (UNIT= 9)
            CLOSE (UNIT=10)
         ENDIF

         PRINT *, 'PFT finished.'
      endif

c     -------------------- EVERYBODY -------------------------------
      deallocate(FIDS, IDS, TS, PS, OS, XS, 
     $     YS, MAGS, PFT_CCS, PRJ_CCS, CMP_CCS)

      call mpi_finalize(mpi_err)
      end program pftsearch
